/* biblio.c */

/* Direttive */
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "listalibri.h"


/* Prototipi delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_libri, char *file_prestiti);

/* Carica un elenco di libri dal file file_libri */
listalibri CaricaLibri (char *file_libri);

/* Stampa l'elenco dei libri disponibili */
void StampaLibri (listalibri L);

/* Esegue i movimenti riportati nel file dei prestiti e dei resi
   sulle liste Scaffale, Prestiti e Resi */
void EsegueMovimenti (char *file_prestiti, listalibri *pScaffale,
                      listalibri *pPrestiti, listalibri *pResi);

/* Cerca il libro AutoreTitolo nella lista L e ne restituisce la posizione
   (NO_ELEMENT se il libro non e' nella lista) */
posizione TrovaLibro (libro AutoreTitolo, listalibri L);

/* Stampa ed esegue le istruzioni per riporre i libri resi nello scaffale */
void RiordinaResi (listalibri *pResi, listalibri *pScaffale);


/* Programma principale */
int main (int argc, char *argv[])
{
  /* Parte dichiarativa */
  char file_libri[ROW_LENGTH];
  char file_prestiti[ROW_LENGTH];
  listalibri Scaffale;
  listalibri Prestiti;
  listalibri Resi;

  /* Parte esecutiva */
  InterpretaLineaComando(argc,argv,file_libri,file_prestiti);

  /* Carica l'elenco dei libri disponibili */
  Scaffale = CaricaLibri(file_libri);

  /* Crea le liste (vuote) dei libri prestati e resi */
  Prestiti = crealista();
  Resi = crealista();

  /* Esegue i movimenti riportati nel file dei prestiti e dei resi */
  EsegueMovimenti(file_prestiti,&Scaffale,&Prestiti,&Resi);

  /* Stampa l'elenco dei libri disponibili */
  StampaLibri(Scaffale);

  /* Stampa ed esegue le istruzioni per riporre i libri resi */
  RiordinaResi(&Resi,&Scaffale);

  /* Stampa l'elenco dei libri disponibili */
  StampaLibri(Scaffale);

  /* Dealloca le strutture dinamiche */
  distruggelista(&Scaffale);
  distruggelista(&Prestiti);
  distruggelista(&Resi);

  return EXIT_SUCCESS;
}


/* Definizione delle procedure secondarie */

void InterpretaLineaComando (int argc, char *argv[], char *file_libri, char *file_prestiti)
{
  if (argc != 3)
  {
    fprintf(stderr,"Errore nella linea di comando!\n");
    exit(EXIT_FAILURE);
  }

  strcpy(file_libri,argv[1]);
  strcpy(file_prestiti,argv[2]);
}


/* Carica un elenco di libri dal file file_libri */
listalibri CaricaLibri (char *file_libri)
{
  FILE *fp;
  listalibri Scaffale;
  char Riga[ROW_LENGTH];
  posizione p;

  /* Apre il file dei libri */
  fp = fopen(file_libri,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",file_libri);
    exit(EXIT_FAILURE);
  }

  /* Crea la lista Scaffale */
  Scaffale = crealista();

  /* Per ogni riga del file */
  while (fscanf(fp,"%[^\n]\n",Riga) == 1)
  {
    /* Se e' il comando di fine, esce dal ciclo */
    if (strcmp(Riga,"FINE") == 0) break;

    /* Se e' un libro, lo aggiunge alla lista Scaffale DOPO l'ultimo elemento */
    p = succlista(Scaffale,ultimolista(Scaffale));
    Scaffale = inslista(Scaffale,p,Riga);
  }

  /* Chiude il file dei libri */
  fclose(fp);

  return Scaffale;
}


/* Stampa l'elenco dei libri disponibili */
void StampaLibri (listalibri L)
{
  posizione p;
  libro AutoreTitolo;

  for (p = primolista(L); !finelista(L,p); p = succlista(L,p))
  {
    leggelista(L,p,AutoreTitolo);
    printf("%s\n",AutoreTitolo);
  }
  printf("FINE\n");
}


/* Esegue i movimenti riportati nel file dei prestiti e dei resi
   sulle liste Scaffale, Prestiti e Resi */
void EsegueMovimenti (char *file_prestiti, listalibri *pScaffale,
                      listalibri *pPrestiti, listalibri *pResi)
{
  FILE *fp;
  char Riga[ROW_LENGTH];
  char Comando[ROW_LENGTH];
  libro AutoreTitolo;
  posizione p;


  /* Apre il file dei movimenti */
  fp = fopen(file_prestiti,"r");
  if (fp == NULL)
  {
    fprintf(stderr,"Errore nell'apertura del file %s!\n",file_prestiti);
    exit(EXIT_FAILURE);
  }

  /* Legge la prima riga saltando l'a capo finale */
  fscanf(fp,"%[^\n]\n",Riga);

  /* Per ogni riga fino al comando FINE */
  while (strcmp(Riga,"FINE") != 0)
  {
    /* Separa il comando e il libro */
    sscanf(Riga,"%s %[^\n]",Comando,AutoreTitolo);

    /* Se il comando e' PRESTITO */
    if (strcmp(Comando,"PRESTITO") == 0)
    {
      /* Cerca il libro sullo Scaffale */
      p = TrovaLibro(AutoreTitolo,*pScaffale);

      /* Se non c'e', termina il programma */
      if (p == NO_ELEMENT)
      {
        fprintf(stderr,"Il libro %s non appartiene alla lista Scaffale!\n",AutoreTitolo);
        exit(EXIT_FAILURE);
      }
      /* Altrimenti sposta il libro da Scaffale a Prestiti */
      *pScaffale = canclista(*pScaffale,&p);
      p = succlista(*pPrestiti,ultimolista(*pPrestiti));
      *pPrestiti = inslista(*pPrestiti,p,AutoreTitolo);
    }
    /* Altrimenti, e' RESTITUZIONE */
    else /* if (strcmp(Comando,"RESTITUZIONE") == 0) */
    {
      /* Cerca il libro nei Prestiti */
      p = TrovaLibro(AutoreTitolo,*pPrestiti);

      /* Se lo trova lo cancella, altrimenti non fa nulla */
      if (p != NO_ELEMENT) canclista(*pPrestiti,&p);

      /* Inserisce il nuovo libro in cima ai Resi */
      p = primolista(*pResi);
      *pResi = inslista(*pResi,p,AutoreTitolo);
    }

    /* Legge la riga seguente */
    fscanf(fp,"%[^\n]\n",Riga);
  }

  /* Chiude il file dei movimenti */
  fclose(fp);
}


/* Cerca il libro AutoreTitolo nella lista L e ne restituisce la posizione
   (NO_ELEMENT se il libro non e' nella lista) */
posizione TrovaLibro (libro AutoreTitolo, listalibri L)
{
  posizione p;
  libro AutoreTitolo2;

  for (p = primolista(L); !finelista(L,p); p = succlista(L,p))
  {
    leggelista(L,p,AutoreTitolo2);
    if (strcmp(AutoreTitolo,AutoreTitolo2) == 0) return p;
  }

  return NO_ELEMENT;
}


/* Stampa ed esegue le istruzioni per riporre i libri resi nello scaffale */
void RiordinaResi (listalibri *pResi, listalibri *pScaffale)
{
  libro AutoreTitolo, AutoreTitolo2;
  posizione p;


  /* Finche' Resi non e' vuota */
  while (!listavuota(*pResi))
  {
    /* Legge il libro in cima alla lista Resi */
    p = primolista(*pResi);
    leggelista(*pResi,p,AutoreTitolo);

    /* Cancella il libro dai Resi */
    *pResi = canclista(*pResi,&p);

    /* Cerca la posizione corretta (primo libro successivo) per i libro AutoreTitolo nello Scaffale */
    for (p = primolista(*pScaffale); !finelista(*pScaffale,p); p = succlista(*pScaffale,p))
    {
      leggelista(*pScaffale,p,AutoreTitolo2);
      if (strcmp(AutoreTitolo,AutoreTitolo2) < 0) break;
    }

    /* Inserisce il libro AutoreTitolo nello Scaffale nella posizione corretta */
    inslista(*pScaffale,p,AutoreTitolo);

    /* Stampa l'istruzione corretta */
    if (finelista(*pScaffale,p))
      printf("METTERE %s PER ULTIMO\n",AutoreTitolo);
    else
      printf("METTERE %s PRIMA DI %s\n",AutoreTitolo,AutoreTitolo2);
  }
  printf("FINE\n");
}
